#!/bin/ksh
#
# Copyright (C) Tivoli Systems/IBM. <2000>
# Unpublished Work
# All Rights Reserved
# Licensed Material - Property of Tivoli Systems/IBM.
#

##############################################################################
# SCRIPT NAME: kossdump.sh
##############################################################################
#
# DESCRIPTION:
#
# This script will generate the output file specified by the third parameter,
# which will contain basic info from the system crash that occurred.  Info
# includes:
#    Name of the system, and OS level
#    The message buffer at the time of the crash
#    Stack of the asserting thread
#    A dump of all thread stacks in the system crash
#    A full stack trace of the asserting thread (with registers)
# 
##############################################################################

KERNEL=""
IMAGE=""
PDOSBIN="opt/pdos/bin"
KERNBIN="/opt/pdos/kernel"
PDOSINST="/opt/pdos"

##############################################################################
# FUNCTION: usage
##############################################################################
#
# This function prints the proper usage of the script.
#
##############################################################################
usage() {
  print "ERROR:Unknown option/bad syntax:"
  print "usage: kossdump.sh [-k kernel_symbols] [-d dump_image]"
  exit 1
}

##############################################################################
# FUNCTION: process_args
##############################################################################
#
# This function processes the command line args, displaying any relevant
# error messages.
#
##############################################################################
process_args () {

  while getopts ":k:d:" opt; do
    case $opt in
      k )
        KERNEL=$OPTARG ;;
      d )  
        IMAGE=$OPTARG ;;
      \? ) usage ;;
    esac
  done
  shift $(($OPTIND - 1))

  if [[ -z $KERNEL ]]; then
    print "DEFAULT kernel symbol table /dev/ksyms!:"
    KERNEL=/dev/ksyms
  fi
  
  if [[ -z $IMAGE ]]; then
    print "DEFAULT for system image is on live system!"
  fi

}

process_args $@

adb -k $KERNEL $IMAGE << EOF
="Initial Dump Information"
="========================"
$<<utsname
srpc_domain/s15t"Domain name"
time/Y15t"Time of crash"
lbolt>a
*time-(*<a%0t100)=Y15t"Time of boot"
*panicstr/n"Panic string:"ts
=n"Message Buffer:"
$<<msgbuf
=n"Stack traceback"
=n
$ C
=n"All running threads:"
$<<threadlist
=n"Full stack traceback with registers:"
<sp$<<stacktrace
=n"VFS info:"
$<<mount
EOF

#
print "End of adb gathered data"
print "====================================================="

#
# Data from the kosseal error logs
#
print "====================================================="
print "====================================================="
print "\nkosserrs.log:"
$KERNBIN/kosserrs
#

if [ -s /var/pdos/log/kosserrs.log.old ] ; then
    print "\nprevious kosserrs.log"
    $KERNBIN/kosserrs -f /var/pdos/log/kosserrs.log.old
fi

#                                                          #
############################################################
# Code added for Defect 2974.                              #
############################################################

############################################################
# Check to see what the kernel symbol table specified was. #
# If it was /dev/ksyms, then running on a live system and  #
# gather live sys info.                                    #
############################################################
if [[ -z $IMAGE || $IMAGE == "/dev/mem" ]]; then
    print "GATHER live system data."
    print "\n\n"
    print "====================================================="
    print "====================================================="
    print "Current running processes (ps -elf)"
    print "====================================================="
    /usr/bin/ps -elf | sort -n +3
    print "====================================================="
    print "====================================================="
    print "Loaded drivers:"
    modinfo
    print "====================================================="
    print "====================================================="
    print "Kosseal kernel trace data:"
    print "====================================================="
    ${KERNBIN}/kazntrace -l
    ${KERNBIN}/kazntrace

    # get statistics
    print "====================================================="
    print "====================================================="
    print "Kosseal kernel statistics data"
    print "====================================================="
    ${KERNBIN}/kossinfo
    print ""
    ${KERNBIN}/kossinfo -x
fi

print "\n\n====================================================="
print "====================================================="
print "System and software info"
print "====================================================="
showrev -a
print -n "System conf: \n"
prtconf -P
print -n "System definition: \n"
sysdef -D $KERNEL
print "Network configuration:"
print "\tnetstat"
netstat -in
print "\tifconfig"
ifconfig -a
print "\nSystem (/etc/system) configuration parameters:"
cat /etc/system | egrep -v  ^[*] | egrep [a-zA-Z]
print ""

print "\n\n====================================================="
print "s/w version data"
OSLEVEL=`uname -r`
if [[ $OSLEVEL == "5.7" || $OSLEVEL == "5.8" ]] ; then
    # Check the arch support
    isainfo | grep sparcv9 > /dev/null
    if [ $? -eq 0 ] ; then
	ARCH="sparcv9/"
	if [[ $OSLEVEL == "5.7" ]] ; then
	    BITS=64_27
	else
	    BITS=64_28
	fi
    else
	if [[ $OSLEVEL == "5.7" ]] ; then
	    BITS=32_27
	else
	    BITS=32_28
	fi
    fi
else
    BITS=32_26
fi
print -n "/usr/kernel/drv/${ARCH}kazndrv sum: "
print "`sum  ${PDOSINST}/kernel/kazndrv`"
print -n "/usr/kernel/strmod/${ARCH}kazntli sum: "
print "`sum ${PDOSINST}/kernel/kazntli`"
print -n "${PDOSBIN}/pdosd: "
print "`sum ${PDOSBIN}/pdosd`"
if [ -s /usr/ccs/bin/what ] ; then
    /usr/ccs/bin/what ${PDOSINST}/kernel/kazndrv${BITS}
    /usr/ccs/bin/what ${PDOSINST}/kernel/kazntli${BITS}
    /usr/ccs/bin/what ${PDOSBIN}/pdosd
fi

exit 0
